"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CONFIG = exports.MathJax = exports.Loader = exports.PathFilters = exports.PackageError = exports.Package = void 0;
const global_js_1 = require("./global.js");
const package_js_1 = require("./package.js");
var package_js_2 = require("./package.js");
Object.defineProperty(exports, "Package", { enumerable: true, get: function () { return package_js_2.Package; } });
Object.defineProperty(exports, "PackageError", { enumerable: true, get: function () { return package_js_2.PackageError; } });
const FunctionList_js_1 = require("../util/FunctionList.js");
exports.PathFilters = {
    source: (data) => {
        if (exports.CONFIG.source.hasOwnProperty(data.name)) {
            data.name = exports.CONFIG.source[data.name];
        }
        return true;
    },
    normalize: (data) => {
        const name = data.name;
        if (!name.match(/^(?:[a-z]+:\/)?\/|[a-z]:\\|\[/i)) {
            data.name = '[mathjax]/' + name.replace(/^\.\//, '');
        }
        if (data.addExtension && !name.match(/\.[^\/]+$/)) {
            data.name += '.js';
        }
        return true;
    },
    prefix: (data) => {
        let match;
        while ((match = data.name.match(/^\[([^\]]*)\]/))) {
            if (!exports.CONFIG.paths.hasOwnProperty(match[1]))
                break;
            data.name = exports.CONFIG.paths[match[1]] + data.name.substr(match[0].length);
        }
        return true;
    }
};
var Loader;
(function (Loader) {
    const VERSION = global_js_1.MathJax.version;
    Loader.versions = new Map();
    function ready(...names) {
        if (names.length === 0) {
            names = Array.from(package_js_1.Package.packages.keys());
        }
        const promises = [];
        for (const name of names) {
            const extension = package_js_1.Package.packages.get(name) || new package_js_1.Package(name, true);
            promises.push(extension.promise);
        }
        return Promise.all(promises);
    }
    Loader.ready = ready;
    function load(...names) {
        if (names.length === 0) {
            return Promise.resolve();
        }
        const promises = [];
        for (const name of names) {
            let extension = package_js_1.Package.packages.get(name);
            if (!extension) {
                extension = new package_js_1.Package(name);
                extension.provides(exports.CONFIG.provides[name]);
            }
            extension.checkNoLoad();
            promises.push(extension.promise.then(() => {
                if (!exports.CONFIG.versionWarnings)
                    return;
                if (extension.isLoaded && !Loader.versions.has(package_js_1.Package.resolvePath(name))) {
                    console.warn(`No version information available for component ${name}`);
                }
            }));
        }
        package_js_1.Package.loadAll();
        return Promise.all(promises);
    }
    Loader.load = load;
    function preLoad(...names) {
        for (const name of names) {
            let extension = package_js_1.Package.packages.get(name);
            if (!extension) {
                extension = new package_js_1.Package(name, true);
                extension.provides(exports.CONFIG.provides[name]);
            }
            extension.loaded();
        }
    }
    Loader.preLoad = preLoad;
    function defaultReady() {
        if (typeof exports.MathJax.startup !== 'undefined') {
            exports.MathJax.config.startup.ready();
        }
    }
    Loader.defaultReady = defaultReady;
    function getRoot() {
        let root = __dirname + '/../../es5';
        if (typeof document !== 'undefined') {
            const script = document.currentScript || document.getElementById('MathJax-script');
            if (script) {
                root = script.src.replace(/\/[^\/]*$/, '');
            }
        }
        return root;
    }
    Loader.getRoot = getRoot;
    function checkVersion(name, version, _type) {
        Loader.versions.set(package_js_1.Package.resolvePath(name), VERSION);
        if (exports.CONFIG.versionWarnings && version !== VERSION) {
            console.warn(`Component ${name} uses ${version} of MathJax; version in use is ${VERSION}`);
            return true;
        }
        return false;
    }
    Loader.checkVersion = checkVersion;
    Loader.pathFilters = new FunctionList_js_1.FunctionList();
    Loader.pathFilters.add(exports.PathFilters.source, 0);
    Loader.pathFilters.add(exports.PathFilters.normalize, 10);
    Loader.pathFilters.add(exports.PathFilters.prefix, 20);
})(Loader = exports.Loader || (exports.Loader = {}));
exports.MathJax = global_js_1.MathJax;
if (typeof exports.MathJax.loader === 'undefined') {
    (0, global_js_1.combineDefaults)(exports.MathJax.config, 'loader', {
        paths: {
            mathjax: Loader.getRoot()
        },
        source: {},
        dependencies: {},
        provides: {},
        load: [],
        ready: Loader.defaultReady.bind(Loader),
        failed: (error) => console.log(`MathJax(${error.package || '?'}): ${error.message}`),
        require: null,
        pathFilters: [],
        versionWarnings: true
    });
    (0, global_js_1.combineWithMathJax)({
        loader: Loader
    });
    for (const filter of exports.MathJax.config.loader.pathFilters) {
        if (Array.isArray(filter)) {
            Loader.pathFilters.add(filter[0], filter[1]);
        }
        else {
            Loader.pathFilters.add(filter);
        }
    }
}
exports.CONFIG = exports.MathJax.config.loader;
//# sourceMappingURL=loader.js.map