"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CONFIG = exports.MathJax = exports.Startup = void 0;
const global_js_1 = require("./global.js");
const PrioritizedList_js_1 = require("../util/PrioritizedList.js");
const Options_js_1 = require("../util/Options.js");
var Startup;
(function (Startup) {
    const extensions = new PrioritizedList_js_1.PrioritizedList();
    let visitor;
    let mathjax;
    Startup.constructors = {};
    Startup.input = [];
    Startup.output = null;
    Startup.handler = null;
    Startup.adaptor = null;
    Startup.elements = null;
    Startup.document = null;
    Startup.promise = new Promise((resolve, reject) => {
        Startup.promiseResolve = resolve;
        Startup.promiseReject = reject;
    });
    Startup.pagePromise = new Promise((resolve, _reject) => {
        const doc = global.document;
        if (!doc || !doc.readyState || doc.readyState === 'complete' || doc.readyState === 'interactive') {
            resolve();
        }
        else {
            const listener = () => resolve();
            doc.defaultView.addEventListener('load', listener, true);
            doc.defaultView.addEventListener('DOMContentLoaded', listener, true);
        }
    });
    function toMML(node) {
        return visitor.visitTree(node, Startup.document);
    }
    Startup.toMML = toMML;
    function registerConstructor(name, constructor) {
        Startup.constructors[name] = constructor;
    }
    Startup.registerConstructor = registerConstructor;
    function useHandler(name, force = false) {
        if (!exports.CONFIG.handler || force) {
            exports.CONFIG.handler = name;
        }
    }
    Startup.useHandler = useHandler;
    function useAdaptor(name, force = false) {
        if (!exports.CONFIG.adaptor || force) {
            exports.CONFIG.adaptor = name;
        }
    }
    Startup.useAdaptor = useAdaptor;
    function useInput(name, force = false) {
        if (!inputSpecified || force) {
            exports.CONFIG.input.push(name);
        }
    }
    Startup.useInput = useInput;
    function useOutput(name, force = false) {
        if (!exports.CONFIG.output || force) {
            exports.CONFIG.output = name;
        }
    }
    Startup.useOutput = useOutput;
    function extendHandler(extend, priority = 10) {
        extensions.add(extend, priority);
    }
    Startup.extendHandler = extendHandler;
    function defaultReady() {
        getComponents();
        makeMethods();
        Startup.pagePromise
            .then(() => exports.CONFIG.pageReady())
            .then(() => Startup.promiseResolve())
            .catch((err) => Startup.promiseReject(err));
    }
    Startup.defaultReady = defaultReady;
    function defaultPageReady() {
        return (exports.CONFIG.typeset && exports.MathJax.typesetPromise ?
            exports.MathJax.typesetPromise(exports.CONFIG.elements) :
            Promise.resolve());
    }
    Startup.defaultPageReady = defaultPageReady;
    function getComponents() {
        visitor = new exports.MathJax._.core.MmlTree.SerializedMmlVisitor.SerializedMmlVisitor();
        mathjax = exports.MathJax._.mathjax.mathjax;
        Startup.input = getInputJax();
        Startup.output = getOutputJax();
        Startup.adaptor = getAdaptor();
        if (Startup.handler) {
            mathjax.handlers.unregister(Startup.handler);
        }
        Startup.handler = getHandler();
        if (Startup.handler) {
            mathjax.handlers.register(Startup.handler);
            Startup.document = getDocument();
        }
    }
    Startup.getComponents = getComponents;
    function makeMethods() {
        if (Startup.input && Startup.output) {
            makeTypesetMethods();
        }
        const oname = (Startup.output ? Startup.output.name.toLowerCase() : '');
        for (const jax of Startup.input) {
            const iname = jax.name.toLowerCase();
            makeMmlMethods(iname, jax);
            makeResetMethod(iname, jax);
            if (Startup.output) {
                makeOutputMethods(iname, oname, jax);
            }
        }
    }
    Startup.makeMethods = makeMethods;
    function makeTypesetMethods() {
        exports.MathJax.typeset = (elements = null) => {
            Startup.document.options.elements = elements;
            Startup.document.reset();
            Startup.document.render();
        };
        exports.MathJax.typesetPromise = (elements = null) => {
            Startup.document.options.elements = elements;
            Startup.document.reset();
            return mathjax.handleRetriesFor(() => {
                Startup.document.render();
            });
        };
        exports.MathJax.typesetClear = (elements = null) => {
            if (elements) {
                Startup.document.clearMathItemsWithin(elements);
            }
            else {
                Startup.document.clear();
            }
        };
    }
    Startup.makeTypesetMethods = makeTypesetMethods;
    function makeOutputMethods(iname, oname, input) {
        const name = iname + '2' + oname;
        exports.MathJax[name] =
            (math, options = {}) => {
                options.format = input.name;
                return Startup.document.convert(math, options);
            };
        exports.MathJax[name + 'Promise'] =
            (math, options = {}) => {
                options.format = input.name;
                return mathjax.handleRetriesFor(() => Startup.document.convert(math, options));
            };
        exports.MathJax[oname + 'Stylesheet'] = () => Startup.output.styleSheet(Startup.document);
        if ('getMetricsFor' in Startup.output) {
            exports.MathJax.getMetricsFor = (node, display) => {
                return Startup.output.getMetricsFor(node, display);
            };
        }
    }
    Startup.makeOutputMethods = makeOutputMethods;
    function makeMmlMethods(name, input) {
        const STATE = exports.MathJax._.core.MathItem.STATE;
        exports.MathJax[name + '2mml'] =
            (math, options = {}) => {
                options.end = STATE.CONVERT;
                options.format = input.name;
                return toMML(Startup.document.convert(math, options));
            };
        exports.MathJax[name + '2mmlPromise'] =
            (math, options = {}) => {
                options.end = STATE.CONVERT;
                options.format = input.name;
                return mathjax.handleRetriesFor(() => toMML(Startup.document.convert(math, options)));
            };
    }
    Startup.makeMmlMethods = makeMmlMethods;
    function makeResetMethod(name, input) {
        exports.MathJax[name + 'Reset'] = (...args) => input.reset(...args);
    }
    Startup.makeResetMethod = makeResetMethod;
    function getInputJax() {
        const jax = [];
        for (const name of exports.CONFIG.input) {
            const inputClass = Startup.constructors[name];
            if (inputClass) {
                jax.push(new inputClass(exports.MathJax.config[name]));
            }
            else {
                throw Error('Input Jax "' + name + '" is not defined (has it been loaded?)');
            }
        }
        return jax;
    }
    Startup.getInputJax = getInputJax;
    function getOutputJax() {
        const name = exports.CONFIG.output;
        if (!name)
            return null;
        const outputClass = Startup.constructors[name];
        if (!outputClass) {
            throw Error('Output Jax "' + name + '" is not defined (has it been loaded?)');
        }
        return new outputClass(exports.MathJax.config[name]);
    }
    Startup.getOutputJax = getOutputJax;
    function getAdaptor() {
        const name = exports.CONFIG.adaptor;
        if (!name || name === 'none')
            return null;
        const adaptor = Startup.constructors[name];
        if (!adaptor) {
            throw Error('DOMAdaptor "' + name + '" is not defined (has it been loaded?)');
        }
        return adaptor(exports.MathJax.config[name]);
    }
    Startup.getAdaptor = getAdaptor;
    function getHandler() {
        const name = exports.CONFIG.handler;
        if (!name || name === 'none' || !Startup.adaptor)
            return null;
        const handlerClass = Startup.constructors[name];
        if (!handlerClass) {
            throw Error('Handler "' + name + '" is not defined (has it been loaded?)');
        }
        let handler = new handlerClass(Startup.adaptor, 5);
        for (const extend of extensions) {
            handler = extend.item(handler);
        }
        return handler;
    }
    Startup.getHandler = getHandler;
    function getDocument(root = null) {
        return mathjax.document(root || exports.CONFIG.document, Object.assign(Object.assign({}, exports.MathJax.config.options), { InputJax: Startup.input, OutputJax: Startup.output }));
    }
    Startup.getDocument = getDocument;
})(Startup = exports.Startup || (exports.Startup = {}));
exports.MathJax = global_js_1.MathJax;
if (typeof exports.MathJax._.startup === 'undefined') {
    (0, global_js_1.combineDefaults)(exports.MathJax.config, 'startup', {
        input: [],
        output: '',
        handler: null,
        adaptor: null,
        document: (typeof document === 'undefined' ? '' : document),
        elements: null,
        typeset: true,
        ready: Startup.defaultReady.bind(Startup),
        pageReady: Startup.defaultPageReady.bind(Startup)
    });
    (0, global_js_1.combineWithMathJax)({
        startup: Startup,
        options: {}
    });
    if (exports.MathJax.config.startup.invalidOption) {
        Options_js_1.OPTIONS.invalidOption = exports.MathJax.config.startup.invalidOption;
    }
    if (exports.MathJax.config.startup.optionError) {
        Options_js_1.OPTIONS.optionError = exports.MathJax.config.startup.optionError;
    }
}
exports.CONFIG = exports.MathJax.config.startup;
const inputSpecified = exports.CONFIG.input.length !== 0;
//# sourceMappingURL=startup.js.map