"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SerializedMmlVisitor = exports.toEntity = exports.DATAMJX = void 0;
const MmlVisitor_js_1 = require("./MmlVisitor.js");
const MmlNode_js_1 = require("./MmlNode.js");
const mi_js_1 = require("./MmlNodes/mi.js");
exports.DATAMJX = 'data-mjx-';
const toEntity = (c) => '&#x' + c.codePointAt(0).toString(16).toUpperCase() + ';';
exports.toEntity = toEntity;
class SerializedMmlVisitor extends MmlVisitor_js_1.MmlVisitor {
    visitTree(node) {
        return this.visitNode(node, '');
    }
    visitTextNode(node, _space) {
        return this.quoteHTML(node.getText());
    }
    visitXMLNode(node, space) {
        return space + node.getSerializedXML();
    }
    visitInferredMrowNode(node, space) {
        let mml = [];
        for (const child of node.childNodes) {
            mml.push(this.visitNode(child, space));
        }
        return mml.join('\n');
    }
    visitTeXAtomNode(node, space) {
        let children = this.childNodeMml(node, space + '  ', '\n');
        let mml = space + '<mrow' + this.getAttributes(node) + '>' +
            (children.match(/\S/) ? '\n' + children + space : '') + '</mrow>';
        return mml;
    }
    visitAnnotationNode(node, space) {
        return space + '<annotation' + this.getAttributes(node) + '>'
            + this.childNodeMml(node, '', '')
            + '</annotation>';
    }
    visitDefault(node, space) {
        let kind = node.kind;
        let [nl, endspace] = (node.isToken || node.childNodes.length === 0 ? ['', ''] : ['\n', space]);
        const children = this.childNodeMml(node, space + '  ', nl);
        return space + '<' + kind + this.getAttributes(node) + '>'
            + (children.match(/\S/) ? nl + children + endspace : '')
            + '</' + kind + '>';
    }
    childNodeMml(node, space, nl) {
        let mml = '';
        for (const child of node.childNodes) {
            mml += this.visitNode(child, space) + nl;
        }
        return mml;
    }
    getAttributes(node) {
        const attr = [];
        const defaults = this.constructor.defaultAttributes[node.kind] || {};
        const attributes = Object.assign({}, defaults, this.getDataAttributes(node), node.attributes.getAllAttributes());
        const variants = this.constructor.variants;
        if (attributes.hasOwnProperty('mathvariant') && variants.hasOwnProperty(attributes.mathvariant)) {
            attributes.mathvariant = variants[attributes.mathvariant];
        }
        for (const name of Object.keys(attributes)) {
            const value = String(attributes[name]);
            if (value === undefined)
                continue;
            attr.push(name + '="' + this.quoteHTML(value) + '"');
        }
        return attr.length ? ' ' + attr.join(' ') : '';
    }
    getDataAttributes(node) {
        const data = {};
        const variant = node.attributes.getExplicit('mathvariant');
        const variants = this.constructor.variants;
        variant && variants.hasOwnProperty(variant) && this.setDataAttribute(data, 'variant', variant);
        node.getProperty('variantForm') && this.setDataAttribute(data, 'alternate', '1');
        node.getProperty('pseudoscript') && this.setDataAttribute(data, 'pseudoscript', 'true');
        node.getProperty('autoOP') === false && this.setDataAttribute(data, 'auto-op', 'false');
        const scriptalign = node.getProperty('scriptalign');
        scriptalign && this.setDataAttribute(data, 'script-align', scriptalign);
        const texclass = node.getProperty('texClass');
        if (texclass !== undefined) {
            let setclass = true;
            if (texclass === MmlNode_js_1.TEXCLASS.OP && node.isKind('mi')) {
                const name = node.getText();
                setclass = !(name.length > 1 && name.match(mi_js_1.MmlMi.operatorName));
            }
            setclass && this.setDataAttribute(data, 'texclass', texclass < 0 ? 'NONE' : MmlNode_js_1.TEXCLASSNAMES[texclass]);
        }
        node.getProperty('scriptlevel') && node.getProperty('useHeight') === false &&
            this.setDataAttribute(data, 'smallmatrix', 'true');
        return data;
    }
    setDataAttribute(data, name, value) {
        data[exports.DATAMJX + name] = value;
    }
    quoteHTML(value) {
        return value
            .replace(/&/g, '&amp;')
            .replace(/</g, '&lt;').replace(/>/g, '&gt;')
            .replace(/\"/g, '&quot;')
            .replace(/[\uD800-\uDBFF]./g, exports.toEntity)
            .replace(/[\u0080-\uD7FF\uE000-\uFFFF]/g, exports.toEntity);
    }
}
exports.SerializedMmlVisitor = SerializedMmlVisitor;
SerializedMmlVisitor.variants = {
    '-tex-calligraphic': 'script',
    '-tex-bold-calligraphic': 'bold-script',
    '-tex-oldstyle': 'normal',
    '-tex-bold-oldstyle': 'bold',
    '-tex-mathit': 'italic'
};
SerializedMmlVisitor.defaultAttributes = {
    math: {
        xmlns: 'http://www.w3.org/1998/Math/MathML'
    }
};
//# sourceMappingURL=SerializedMmlVisitor.js.map