"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HTMLDocument = void 0;
const MathDocument_js_1 = require("../../core/MathDocument.js");
const Options_js_1 = require("../../util/Options.js");
const HTMLMathItem_js_1 = require("./HTMLMathItem.js");
const HTMLMathList_js_1 = require("./HTMLMathList.js");
const HTMLDomStrings_js_1 = require("./HTMLDomStrings.js");
const MathItem_js_1 = require("../../core/MathItem.js");
class HTMLDocument extends MathDocument_js_1.AbstractMathDocument {
    constructor(document, adaptor, options) {
        let [html, dom] = (0, Options_js_1.separateOptions)(options, HTMLDomStrings_js_1.HTMLDomStrings.OPTIONS);
        super(document, adaptor, html);
        this.domStrings = this.options['DomStrings'] || new HTMLDomStrings_js_1.HTMLDomStrings(dom);
        this.domStrings.adaptor = adaptor;
        this.styles = [];
    }
    findPosition(N, index, delim, nodes) {
        const adaptor = this.adaptor;
        for (const list of nodes[N]) {
            let [node, n] = list;
            if (index <= n && adaptor.kind(node) === '#text') {
                return { node: node, n: Math.max(index, 0), delim: delim };
            }
            index -= n;
        }
        return { node: null, n: 0, delim: delim };
    }
    mathItem(item, jax, nodes) {
        let math = item.math;
        let start = this.findPosition(item.n, item.start.n, item.open, nodes);
        let end = this.findPosition(item.n, item.end.n, item.close, nodes);
        return new this.options.MathItem(math, jax, item.display, start, end);
    }
    findMath(options) {
        if (!this.processed.isSet('findMath')) {
            this.adaptor.document = this.document;
            options = (0, Options_js_1.userOptions)({ elements: this.options.elements || [this.adaptor.body(this.document)] }, options);
            for (const container of this.adaptor.getElements(options['elements'], this.document)) {
                let [strings, nodes] = [null, null];
                for (const jax of this.inputJax) {
                    let list = new (this.options['MathList'])();
                    if (jax.processStrings) {
                        if (strings === null) {
                            [strings, nodes] = this.domStrings.find(container);
                        }
                        for (const math of jax.findMath(strings)) {
                            list.push(this.mathItem(math, jax, nodes));
                        }
                    }
                    else {
                        for (const math of jax.findMath(container)) {
                            let item = new this.options.MathItem(math.math, jax, math.display, math.start, math.end);
                            list.push(item);
                        }
                    }
                    this.math.merge(list);
                }
            }
            this.processed.set('findMath');
        }
        return this;
    }
    updateDocument() {
        if (!this.processed.isSet('updateDocument')) {
            this.addPageElements();
            this.addStyleSheet();
            super.updateDocument();
            this.processed.set('updateDocument');
        }
        return this;
    }
    addPageElements() {
        const body = this.adaptor.body(this.document);
        const node = this.documentPageElements();
        if (node) {
            this.adaptor.append(body, node);
        }
    }
    addStyleSheet() {
        const sheet = this.documentStyleSheet();
        const adaptor = this.adaptor;
        if (sheet && !adaptor.parent(sheet)) {
            const head = adaptor.head(this.document);
            let styles = this.findSheet(head, adaptor.getAttribute(sheet, 'id'));
            if (styles) {
                adaptor.replace(sheet, styles);
            }
            else {
                adaptor.append(head, sheet);
            }
        }
    }
    findSheet(head, id) {
        if (id) {
            for (const sheet of this.adaptor.tags(head, 'style')) {
                if (this.adaptor.getAttribute(sheet, 'id') === id) {
                    return sheet;
                }
            }
        }
        return null;
    }
    removeFromDocument(restore = false) {
        if (this.processed.isSet('updateDocument')) {
            for (const math of this.math) {
                if (math.state() >= MathItem_js_1.STATE.INSERTED) {
                    math.state(MathItem_js_1.STATE.TYPESET, restore);
                }
            }
        }
        this.processed.clear('updateDocument');
        return this;
    }
    documentStyleSheet() {
        return this.outputJax.styleSheet(this);
    }
    documentPageElements() {
        return this.outputJax.pageElements(this);
    }
    addStyles(styles) {
        this.styles.push(styles);
    }
    getStyles() {
        return this.styles;
    }
}
exports.HTMLDocument = HTMLDocument;
HTMLDocument.KIND = 'HTML';
HTMLDocument.OPTIONS = Object.assign(Object.assign({}, MathDocument_js_1.AbstractMathDocument.OPTIONS), { renderActions: (0, Options_js_1.expandable)(Object.assign(Object.assign({}, MathDocument_js_1.AbstractMathDocument.OPTIONS.renderActions), { styles: [MathItem_js_1.STATE.INSERTED + 1, '', 'updateStyleSheet', false] })), MathList: HTMLMathList_js_1.HTMLMathList, MathItem: HTMLMathItem_js_1.HTMLMathItem, DomStrings: null });
//# sourceMappingURL=HTMLDocument.js.map