"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TeX = void 0;
const InputJax_js_1 = require("../core/InputJax.js");
const Options_js_1 = require("../util/Options.js");
const FindTeX_js_1 = require("./tex/FindTeX.js");
const FilterUtil_js_1 = __importDefault(require("./tex/FilterUtil.js"));
const NodeUtil_js_1 = __importDefault(require("./tex/NodeUtil.js"));
const TexParser_js_1 = __importDefault(require("./tex/TexParser.js"));
const TexError_js_1 = __importDefault(require("./tex/TexError.js"));
const ParseOptions_js_1 = __importDefault(require("./tex/ParseOptions.js"));
const Tags_js_1 = require("./tex/Tags.js");
const Configuration_js_1 = require("./tex/Configuration.js");
require("./tex/base/BaseConfiguration.js");
class TeX extends InputJax_js_1.AbstractInputJax {
    static configure(packages) {
        let configuration = new Configuration_js_1.ParserConfiguration(packages, ['tex']);
        configuration.init();
        return configuration;
    }
    static tags(options, configuration) {
        Tags_js_1.TagsFactory.addTags(configuration.tags);
        Tags_js_1.TagsFactory.setDefault(options.options.tags);
        options.tags = Tags_js_1.TagsFactory.getDefault();
        options.tags.configuration = options;
    }
    constructor(options = {}) {
        const [rest, tex, find] = (0, Options_js_1.separateOptions)(options, TeX.OPTIONS, FindTeX_js_1.FindTeX.OPTIONS);
        super(tex);
        this.findTeX = this.options['FindTeX'] || new FindTeX_js_1.FindTeX(find);
        const packages = this.options.packages;
        const configuration = this.configuration = TeX.configure(packages);
        const parseOptions = this._parseOptions =
            new ParseOptions_js_1.default(configuration, [this.options, Tags_js_1.TagsFactory.OPTIONS]);
        (0, Options_js_1.userOptions)(parseOptions.options, rest);
        configuration.config(this);
        TeX.tags(parseOptions, configuration);
        this.postFilters.add(FilterUtil_js_1.default.cleanSubSup, -6);
        this.postFilters.add(FilterUtil_js_1.default.setInherited, -5);
        this.postFilters.add(FilterUtil_js_1.default.moveLimits, -4);
        this.postFilters.add(FilterUtil_js_1.default.cleanStretchy, -3);
        this.postFilters.add(FilterUtil_js_1.default.cleanAttributes, -2);
        this.postFilters.add(FilterUtil_js_1.default.combineRelations, -1);
    }
    setMmlFactory(mmlFactory) {
        super.setMmlFactory(mmlFactory);
        this._parseOptions.nodeFactory.setMmlFactory(mmlFactory);
    }
    get parseOptions() {
        return this._parseOptions;
    }
    reset(tag = 0) {
        this.parseOptions.tags.reset(tag);
    }
    compile(math, document) {
        this.parseOptions.clear();
        this.executeFilters(this.preFilters, math, document, this.parseOptions);
        let display = math.display;
        this.latex = math.math;
        let node;
        this.parseOptions.tags.startEquation(math);
        let globalEnv;
        try {
            let parser = new TexParser_js_1.default(this.latex, { display: display, isInner: false }, this.parseOptions);
            node = parser.mml();
            globalEnv = parser.stack.global;
        }
        catch (err) {
            if (!(err instanceof TexError_js_1.default)) {
                throw err;
            }
            this.parseOptions.error = true;
            node = this.options.formatError(this, err);
        }
        node = this.parseOptions.nodeFactory.create('node', 'math', [node]);
        if (globalEnv === null || globalEnv === void 0 ? void 0 : globalEnv.indentalign) {
            NodeUtil_js_1.default.setAttribute(node, 'indentalign', globalEnv.indentalign);
        }
        if (display) {
            NodeUtil_js_1.default.setAttribute(node, 'display', 'block');
        }
        this.parseOptions.tags.finishEquation(math);
        this.parseOptions.root = node;
        this.executeFilters(this.postFilters, math, document, this.parseOptions);
        this.mathNode = this.parseOptions.root;
        return this.mathNode;
    }
    findMath(strings) {
        return this.findTeX.findMath(strings);
    }
    formatError(err) {
        let message = err.message.replace(/\n.*/, '');
        return this.parseOptions.nodeFactory.create('error', message, err.id, this.latex);
    }
}
exports.TeX = TeX;
TeX.NAME = 'TeX';
TeX.OPTIONS = Object.assign(Object.assign({}, InputJax_js_1.AbstractInputJax.OPTIONS), { FindTeX: null, packages: ['base'], digits: /^(?:[0-9]+(?:\{,\}[0-9]{3})*(?:\.[0-9]*)?|\.[0-9]+)/, maxBuffer: 5 * 1024, formatError: (jax, err) => jax.formatError(err) });
//# sourceMappingURL=tex.js.map