"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommonOutputJax = void 0;
const OutputJax_js_1 = require("../../core/OutputJax.js");
const MathItem_js_1 = require("../../core/MathItem.js");
const Options_js_1 = require("../../util/Options.js");
const lengths_js_1 = require("../../util/lengths.js");
const Styles_js_1 = require("../../util/Styles.js");
const StyleList_js_1 = require("../../util/StyleList.js");
class CommonOutputJax extends OutputJax_js_1.AbstractOutputJax {
    constructor(options = null, defaultFactory = null, defaultFont = null) {
        const [jaxOptions, fontOptions] = (0, Options_js_1.separateOptions)(options, defaultFont.OPTIONS);
        super(jaxOptions);
        this.factory = this.options.wrapperFactory ||
            new defaultFactory();
        this.factory.jax = this;
        this.cssStyles = this.options.cssStyles || new StyleList_js_1.CssStyles();
        this.font = this.options.font || new defaultFont(fontOptions);
        this.unknownCache = new Map();
    }
    typeset(math, html) {
        this.setDocument(html);
        let node = this.createNode();
        this.toDOM(math, node, html);
        return node;
    }
    createNode() {
        const jax = this.constructor.NAME;
        return this.html('mjx-container', { 'class': 'MathJax', jax: jax });
    }
    setScale(node) {
        const scale = this.math.metrics.scale * this.options.scale;
        if (scale !== 1) {
            this.adaptor.setStyle(node, 'fontSize', (0, lengths_js_1.percent)(scale));
        }
    }
    toDOM(math, node, html = null) {
        this.setDocument(html);
        this.math = math;
        this.pxPerEm = math.metrics.ex / this.font.params.x_height;
        math.root.setTeXclass(null);
        this.setScale(node);
        this.nodeMap = new Map();
        this.container = node;
        this.processMath(math.root, node);
        this.nodeMap = null;
        this.executeFilters(this.postFilters, math, html, node);
    }
    getBBox(math, html) {
        this.setDocument(html);
        this.math = math;
        math.root.setTeXclass(null);
        this.nodeMap = new Map();
        let bbox = this.factory.wrap(math.root).getOuterBBox();
        this.nodeMap = null;
        return bbox;
    }
    getMetrics(html) {
        this.setDocument(html);
        const adaptor = this.adaptor;
        const maps = this.getMetricMaps(html);
        for (const math of html.math) {
            const parent = adaptor.parent(math.start.node);
            if (math.state() < MathItem_js_1.STATE.METRICS && parent) {
                const map = maps[math.display ? 1 : 0];
                const { em, ex, containerWidth, lineWidth, scale, family } = map.get(parent);
                math.setMetrics(em, ex, containerWidth, lineWidth, scale);
                if (this.options.mtextInheritFont) {
                    math.outputData.mtextFamily = family;
                }
                if (this.options.merrorInheritFont) {
                    math.outputData.merrorFamily = family;
                }
                math.state(MathItem_js_1.STATE.METRICS);
            }
        }
    }
    getMetricsFor(node, display) {
        const getFamily = (this.options.mtextInheritFont || this.options.merrorInheritFont);
        const test = this.getTestElement(node, display);
        const metrics = this.measureMetrics(test, getFamily);
        this.adaptor.remove(test);
        return metrics;
    }
    getMetricMaps(html) {
        const adaptor = this.adaptor;
        const domMaps = [new Map(), new Map()];
        for (const math of html.math) {
            const node = adaptor.parent(math.start.node);
            if (node && math.state() < MathItem_js_1.STATE.METRICS) {
                const map = domMaps[math.display ? 1 : 0];
                if (!map.has(node)) {
                    map.set(node, this.getTestElement(node, math.display));
                }
            }
        }
        const getFamily = this.options.mtextInheritFont || this.options.merrorInheritFont;
        const maps = [new Map(), new Map()];
        for (const i of maps.keys()) {
            for (const node of domMaps[i].keys()) {
                maps[i].set(node, this.measureMetrics(domMaps[i].get(node), getFamily));
            }
        }
        for (const i of maps.keys()) {
            for (const node of domMaps[i].values()) {
                adaptor.remove(node);
            }
        }
        return maps;
    }
    getTestElement(node, display) {
        const adaptor = this.adaptor;
        if (!this.testInline) {
            this.testInline = this.html('mjx-test', { style: {
                    display: 'inline-block',
                    width: '100%',
                    'font-style': 'normal',
                    'font-weight': 'normal',
                    'font-size': '100%',
                    'font-size-adjust': 'none',
                    'text-indent': 0,
                    'text-transform': 'none',
                    'letter-spacing': 'normal',
                    'word-spacing': 'normal',
                    overflow: 'hidden',
                    height: '1px',
                    'margin-right': '-1px'
                } }, [
                this.html('mjx-left-box', { style: {
                        display: 'inline-block',
                        width: 0,
                        'float': 'left'
                    } }),
                this.html('mjx-ex-box', { style: {
                        position: 'absolute',
                        overflow: 'hidden',
                        width: '1px', height: '60ex'
                    } }),
                this.html('mjx-right-box', { style: {
                        display: 'inline-block',
                        width: 0,
                        'float': 'right'
                    } })
            ]);
            this.testDisplay = adaptor.clone(this.testInline);
            adaptor.setStyle(this.testDisplay, 'display', 'table');
            adaptor.setStyle(this.testDisplay, 'margin-right', '');
            adaptor.setStyle(adaptor.firstChild(this.testDisplay), 'display', 'none');
            const right = adaptor.lastChild(this.testDisplay);
            adaptor.setStyle(right, 'display', 'table-cell');
            adaptor.setStyle(right, 'width', '10000em');
            adaptor.setStyle(right, 'float', '');
        }
        return adaptor.append(node, adaptor.clone(display ? this.testDisplay : this.testInline));
    }
    measureMetrics(node, getFamily) {
        const adaptor = this.adaptor;
        const family = (getFamily ? adaptor.fontFamily(node) : '');
        const em = adaptor.fontSize(node);
        const [w, h] = adaptor.nodeSize(adaptor.childNode(node, 1));
        const ex = (w ? h / 60 : em * this.options.exFactor);
        const containerWidth = (!w ? 1000000 : adaptor.getStyle(node, 'display') === 'table' ?
            adaptor.nodeSize(adaptor.lastChild(node))[0] - 1 :
            adaptor.nodeBBox(adaptor.lastChild(node)).left -
                adaptor.nodeBBox(adaptor.firstChild(node)).left - 2);
        const scale = Math.max(this.options.minScale, this.options.matchFontHeight ? ex / this.font.params.x_height / em : 1);
        const lineWidth = 1000000;
        return { em, ex, containerWidth, lineWidth, scale, family };
    }
    styleSheet(html) {
        this.setDocument(html);
        this.cssStyles.clear();
        this.cssStyles.addStyles(this.constructor.commonStyles);
        if ('getStyles' in html) {
            for (const styles of html.getStyles()) {
                this.cssStyles.addStyles(styles);
            }
        }
        this.addWrapperStyles(this.cssStyles);
        this.addFontStyles(this.cssStyles);
        const sheet = this.html('style', { id: 'MJX-styles' }, [this.text('\n' + this.cssStyles.cssText + '\n')]);
        return sheet;
    }
    addFontStyles(styles) {
        styles.addStyles(this.font.styles);
    }
    addWrapperStyles(styles) {
        for (const kind of this.factory.getKinds()) {
            this.addClassStyles(this.factory.getNodeClass(kind), styles);
        }
    }
    addClassStyles(CLASS, styles) {
        styles.addStyles(CLASS.styles);
    }
    setDocument(html) {
        if (html) {
            this.document = html;
            this.adaptor.document = html.document;
        }
    }
    html(type, def = {}, content = [], ns) {
        return this.adaptor.node(type, def, content, ns);
    }
    text(text) {
        return this.adaptor.text(text);
    }
    fixed(m, n = 3) {
        if (Math.abs(m) < .0006) {
            return '0';
        }
        return m.toFixed(n).replace(/\.?0+$/, '');
    }
    measureText(text, variant, font = ['', false, false]) {
        const node = this.unknownText(text, variant);
        if (variant === '-explicitFont') {
            const styles = this.cssFontStyles(font);
            this.adaptor.setAttributes(node, { style: styles });
        }
        return this.measureTextNodeWithCache(node, text, variant, font);
    }
    measureTextNodeWithCache(text, chars, variant, font = ['', false, false]) {
        if (variant === '-explicitFont') {
            variant = [font[0], font[1] ? 'T' : 'F', font[2] ? 'T' : 'F', ''].join('-');
        }
        if (!this.unknownCache.has(variant)) {
            this.unknownCache.set(variant, new Map());
        }
        const map = this.unknownCache.get(variant);
        const cached = map.get(chars);
        if (cached)
            return cached;
        const bbox = this.measureTextNode(text);
        map.set(chars, bbox);
        return bbox;
    }
    measureXMLnode(xml) {
        const adaptor = this.adaptor;
        const content = this.html('mjx-xml-block', { style: { display: 'inline-block' } }, [adaptor.clone(xml)]);
        const base = this.html('mjx-baseline', { style: { display: 'inline-block', width: 0, height: 0 } });
        const style = {
            position: 'absolute',
            display: 'inline-block',
            'font-family': 'initial',
            'line-height': 'normal'
        };
        const node = this.html('mjx-measure-xml', { style }, [base, content]);
        adaptor.append(adaptor.parent(this.math.start.node), this.container);
        adaptor.append(this.container, node);
        const em = this.math.metrics.em * this.math.metrics.scale;
        const { left, right, bottom, top } = adaptor.nodeBBox(content);
        const w = (right - left) / em;
        const h = (adaptor.nodeBBox(base).top - top) / em;
        const d = (bottom - top) / em - h;
        adaptor.remove(this.container);
        adaptor.remove(node);
        return { w, h, d };
    }
    cssFontStyles(font, styles = {}) {
        const [family, italic, bold] = font;
        styles['font-family'] = this.font.getFamily(family);
        if (italic)
            styles['font-style'] = 'italic';
        if (bold)
            styles['font-weight'] = 'bold';
        return styles;
    }
    getFontData(styles) {
        if (!styles) {
            styles = new Styles_js_1.Styles();
        }
        return [this.font.getFamily(styles.get('font-family')),
            styles.get('font-style') === 'italic',
            styles.get('font-weight') === 'bold'];
    }
}
exports.CommonOutputJax = CommonOutputJax;
CommonOutputJax.NAME = 'Common';
CommonOutputJax.OPTIONS = Object.assign(Object.assign({}, OutputJax_js_1.AbstractOutputJax.OPTIONS), { scale: 1, minScale: .5, mtextInheritFont: false, merrorInheritFont: false, mtextFont: '', merrorFont: 'serif', mathmlSpacing: false, skipAttributes: {}, exFactor: .5, displayAlign: 'center', displayIndent: '0', wrapperFactory: null, font: null, cssStyles: null });
CommonOutputJax.commonStyles = {};
//# sourceMappingURL=OutputJax.js.map