"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SVG = exports.XLINKNS = exports.SVGNS = void 0;
const OutputJax_js_1 = require("./common/OutputJax.js");
const WrapperFactory_js_1 = require("./svg/WrapperFactory.js");
const tex_js_1 = require("./svg/fonts/tex.js");
const FontCache_js_1 = require("./svg/FontCache.js");
const string_js_1 = require("../util/string.js");
const lengths_js_1 = require("../util/lengths.js");
exports.SVGNS = 'http://www.w3.org/2000/svg';
exports.XLINKNS = 'http://www.w3.org/1999/xlink';
class SVG extends OutputJax_js_1.CommonOutputJax {
    constructor(options = null) {
        super(options, WrapperFactory_js_1.SVGWrapperFactory, tex_js_1.TeXFont);
        this.minwidth = 0;
        this.shift = 0;
        this.container = null;
        this.svgStyles = null;
        this.fontCache = new FontCache_js_1.FontCache(this);
    }
    initialize() {
        if (this.options.fontCache === 'global') {
            this.fontCache.clearCache();
        }
    }
    clearFontCache() {
        this.fontCache.clearCache();
    }
    reset() {
        this.clearFontCache();
    }
    setScale(node) {
        if (this.options.scale !== 1) {
            this.adaptor.setStyle(node, 'fontSize', (0, lengths_js_1.percent)(this.options.scale));
        }
    }
    escaped(math, html) {
        this.setDocument(html);
        return this.html('span', {}, [this.text(math.math)]);
    }
    styleSheet(html) {
        if (this.svgStyles) {
            return this.svgStyles;
        }
        const sheet = this.svgStyles = super.styleSheet(html);
        this.adaptor.setAttribute(sheet, 'id', SVG.STYLESHEETID);
        return sheet;
    }
    pageElements(html) {
        if (this.options.fontCache === 'global' && !this.findCache(html)) {
            return this.svg('svg', { id: SVG.FONTCACHEID, style: { display: 'none' } }, [this.fontCache.getCache()]);
        }
        return null;
    }
    findCache(html) {
        const adaptor = this.adaptor;
        const svgs = adaptor.tags(adaptor.body(html.document), 'svg');
        for (let i = svgs.length - 1; i >= 0; i--) {
            if (this.adaptor.getAttribute(svgs[i], 'id') === SVG.FONTCACHEID) {
                return true;
            }
        }
        return false;
    }
    processMath(math, parent) {
        const container = this.container;
        this.container = parent;
        const wrapper = this.factory.wrap(math);
        const [svg, g] = this.createRoot(wrapper);
        this.typesetSVG(wrapper, svg, g);
        this.container = container;
    }
    createRoot(wrapper) {
        const { w, h, d, pwidth } = wrapper.getOuterBBox();
        const px = wrapper.metrics.em / 1000;
        const W = Math.max(w, px);
        const H = Math.max(h + d, px);
        const g = this.svg('g', {
            stroke: 'currentColor', fill: 'currentColor',
            'stroke-width': 0, transform: 'scale(1,-1)'
        });
        const adaptor = this.adaptor;
        const svg = adaptor.append(this.container, this.svg('svg', {
            xmlns: exports.SVGNS,
            width: this.ex(W), height: this.ex(H),
            role: 'img', focusable: false,
            style: { 'vertical-align': this.ex(-d) },
            viewBox: [0, this.fixed(-h * 1000, 1), this.fixed(W * 1000, 1), this.fixed(H * 1000, 1)].join(' ')
        }, [g]));
        if (W === .001) {
            adaptor.setAttribute(svg, 'preserveAspectRatio', 'xMidYMid slice');
            if (w < 0) {
                adaptor.setStyle(this.container, 'margin-right', this.ex(w));
            }
        }
        if (pwidth) {
            adaptor.setStyle(svg, 'min-width', this.ex(W));
            adaptor.setAttribute(svg, 'width', pwidth);
            adaptor.removeAttribute(svg, 'viewBox');
            const scale = this.fixed(wrapper.metrics.ex / (this.font.params.x_height * 1000), 6);
            adaptor.setAttribute(g, 'transform', `scale(${scale},-${scale}) translate(0, ${this.fixed(-h * 1000, 1)})`);
        }
        if (this.options.fontCache !== 'none') {
            adaptor.setAttribute(svg, 'xmlns:xlink', exports.XLINKNS);
        }
        return [svg, g];
    }
    typesetSVG(wrapper, svg, g) {
        const adaptor = this.adaptor;
        this.minwidth = this.shift = 0;
        if (this.options.fontCache === 'local') {
            this.fontCache.clearCache();
            this.fontCache.useLocalID(this.options.localID);
            adaptor.insert(this.fontCache.getCache(), g);
        }
        wrapper.toSVG(g);
        this.fontCache.clearLocalID();
        if (this.minwidth) {
            adaptor.setStyle(svg, 'minWidth', this.ex(this.minwidth));
            adaptor.setStyle(this.container, 'minWidth', this.ex(this.minwidth));
        }
        else if (this.shift) {
            const align = adaptor.getAttribute(this.container, 'justify') || 'center';
            this.setIndent(svg, align, this.shift);
        }
    }
    setIndent(svg, align, shift) {
        if (align === 'center' || align === 'left') {
            this.adaptor.setStyle(svg, 'margin-left', this.ex(shift));
        }
        if (align === 'center' || align === 'right') {
            this.adaptor.setStyle(svg, 'margin-right', this.ex(-shift));
        }
    }
    ex(m) {
        m /= this.font.params.x_height;
        return (Math.abs(m) < .001 ? '0' : m.toFixed(3).replace(/\.?0+$/, '') + 'ex');
    }
    svg(kind, properties = {}, children = []) {
        return this.html(kind, properties, children, exports.SVGNS);
    }
    unknownText(text, variant) {
        const metrics = this.math.metrics;
        const scale = this.font.params.x_height / metrics.ex * metrics.em * 1000;
        const svg = this.svg('text', {
            'data-variant': variant,
            transform: 'scale(1,-1)', 'font-size': this.fixed(scale, 1) + 'px'
        }, [this.text(text)]);
        const adaptor = this.adaptor;
        if (variant !== '-explicitFont') {
            const c = (0, string_js_1.unicodeChars)(text);
            if (c.length !== 1 || c[0] < 0x1D400 || c[0] > 0x1D7FF) {
                const [family, italic, bold] = this.font.getCssFont(variant);
                adaptor.setAttribute(svg, 'font-family', family);
                if (italic) {
                    adaptor.setAttribute(svg, 'font-style', 'italic');
                }
                if (bold) {
                    adaptor.setAttribute(svg, 'font-weight', 'bold');
                }
            }
        }
        return svg;
    }
    measureTextNode(text) {
        const adaptor = this.adaptor;
        text = adaptor.clone(text);
        adaptor.removeAttribute(text, 'transform');
        const ex = this.fixed(this.font.params.x_height * 1000, 1);
        const svg = this.svg('svg', {
            position: 'absolute', visibility: 'hidden',
            width: '1ex', height: '1ex',
            viewBox: [0, 0, ex, ex].join(' ')
        }, [text]);
        adaptor.append(adaptor.body(adaptor.document), svg);
        let w = adaptor.nodeSize(text, 1000, true)[0];
        adaptor.remove(svg);
        return { w: w, h: .75, d: .2 };
    }
}
exports.SVG = SVG;
SVG.NAME = 'SVG';
SVG.OPTIONS = Object.assign(Object.assign({}, OutputJax_js_1.CommonOutputJax.OPTIONS), { internalSpeechTitles: true, titleID: 0, fontCache: 'local', localID: null });
SVG.commonStyles = {
    'mjx-container[jax="SVG"]': {
        direction: 'ltr'
    },
    'mjx-container[jax="SVG"] > svg': {
        overflow: 'visible',
        'min-height': '1px',
        'min-width': '1px'
    },
    'mjx-container[jax="SVG"] > svg a': {
        fill: 'blue', stroke: 'blue'
    }
};
SVG.FONTCACHEID = 'MJX-SVG-global-cache';
SVG.STYLESHEETID = 'MJX-SVG-styles';
//# sourceMappingURL=svg.js.map