"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SVGWrapper = void 0;
const BBox_js_1 = require("../../util/BBox.js");
const Wrapper_js_1 = require("../common/Wrapper.js");
const svg_js_1 = require("../svg.js");
class SVGWrapper extends Wrapper_js_1.CommonWrapper {
    constructor() {
        super(...arguments);
        this.element = null;
        this.dx = 0;
    }
    toSVG(parent) {
        this.addChildren(this.standardSVGnode(parent));
    }
    addChildren(parent) {
        let x = 0;
        for (const child of this.childNodes) {
            child.toSVG(parent);
            const bbox = child.getOuterBBox();
            if (child.element) {
                child.place(x + bbox.L * bbox.rscale, 0);
            }
            x += (bbox.L + bbox.w + bbox.R) * bbox.rscale;
        }
    }
    standardSVGnode(parent) {
        const svg = this.createSVGnode(parent);
        this.handleStyles();
        this.handleScale();
        this.handleBorder();
        this.handleColor();
        this.handleAttributes();
        return svg;
    }
    createSVGnode(parent) {
        this.element = this.svg('g', { 'data-mml-node': this.node.kind });
        const href = this.node.attributes.get('href');
        if (href) {
            parent = this.adaptor.append(parent, this.svg('a', { href: href }));
            const { h, d, w } = this.getOuterBBox();
            this.adaptor.append(this.element, this.svg('rect', {
                'data-hitbox': true, fill: 'none', stroke: 'none', 'pointer-events': 'all',
                width: this.fixed(w), height: this.fixed(h + d), y: this.fixed(-d)
            }));
        }
        this.adaptor.append(parent, this.element);
        return this.element;
    }
    handleStyles() {
        if (!this.styles)
            return;
        const styles = this.styles.cssText;
        if (styles) {
            this.adaptor.setAttribute(this.element, 'style', styles);
        }
        BBox_js_1.BBox.StyleAdjust.forEach(([name, , lr]) => {
            if (lr !== 0)
                return;
            const x = this.styles.get(name);
            if (x) {
                this.dx += this.length2em(x, 1, this.bbox.rscale);
            }
        });
    }
    handleScale() {
        if (this.bbox.rscale !== 1) {
            const scale = 'scale(' + this.fixed(this.bbox.rscale / 1000, 3) + ')';
            this.adaptor.setAttribute(this.element, 'transform', scale);
        }
    }
    handleColor() {
        var _a;
        const adaptor = this.adaptor;
        const attributes = this.node.attributes;
        const mathcolor = attributes.getExplicit('mathcolor');
        const color = attributes.getExplicit('color');
        const mathbackground = attributes.getExplicit('mathbackground');
        const background = attributes.getExplicit('background');
        const bgcolor = (((_a = this.styles) === null || _a === void 0 ? void 0 : _a.get('background-color')) || '');
        if (mathcolor || color) {
            adaptor.setAttribute(this.element, 'fill', mathcolor || color);
            adaptor.setAttribute(this.element, 'stroke', mathcolor || color);
        }
        if (mathbackground || background || bgcolor) {
            let { h, d, w } = this.getOuterBBox();
            let rect = this.svg('rect', {
                fill: mathbackground || background || bgcolor,
                x: this.fixed(-this.dx), y: this.fixed(-d),
                width: this.fixed(w),
                height: this.fixed(h + d),
                'data-bgcolor': true
            });
            let child = adaptor.firstChild(this.element);
            if (child) {
                adaptor.insert(rect, child);
            }
            else {
                adaptor.append(this.element, rect);
            }
        }
    }
    handleBorder() {
        if (!this.styles)
            return;
        const width = Array(4).fill(0);
        const style = Array(4);
        const color = Array(4);
        for (const [name, i] of [['Top', 0], ['Right', 1], ['Bottom', 2], ['Left', 3]]) {
            const key = 'border' + name;
            const w = this.styles.get(key + 'Width');
            if (!w)
                continue;
            width[i] = Math.max(0, this.length2em(w, 1, this.bbox.rscale));
            style[i] = this.styles.get(key + 'Style') || 'solid';
            color[i] = this.styles.get(key + 'Color') || 'currentColor';
        }
        const f = SVGWrapper.borderFuzz;
        const bbox = this.getOuterBBox();
        const [h, d, w] = [bbox.h + f, bbox.d + f, bbox.w + f];
        const outerRT = [w, h];
        const outerLT = [-f, h];
        const outerRB = [w, -d];
        const outerLB = [-f, -d];
        const innerRT = [w - width[1], h - width[0]];
        const innerLT = [-f + width[3], h - width[0]];
        const innerRB = [w - width[1], -d + width[2]];
        const innerLB = [-f + width[3], -d + width[2]];
        const paths = [
            [outerLT, outerRT, innerRT, innerLT],
            [outerRB, outerRT, innerRT, innerRB],
            [outerLB, outerRB, innerRB, innerLB],
            [outerLB, outerLT, innerLT, innerLB]
        ];
        const adaptor = this.adaptor;
        const child = adaptor.firstChild(this.element);
        for (const i of [0, 1, 2, 3]) {
            if (!width[i])
                continue;
            const path = paths[i];
            if (style[i] === 'dashed' || style[i] === 'dotted') {
                this.addBorderBroken(path, color[i], style[i], width[i], i);
            }
            else {
                this.addBorderSolid(path, color[i], child);
            }
        }
    }
    addBorderSolid(path, color, child) {
        const border = this.svg('polygon', {
            points: path.map(([x, y]) => `${this.fixed(x - this.dx)},${this.fixed(y)}`).join(' '),
            stroke: 'none',
            fill: color
        });
        if (child) {
            this.adaptor.insert(border, child);
        }
        else {
            this.adaptor.append(this.element, border);
        }
    }
    addBorderBroken(path, color, style, t, i) {
        const dot = (style === 'dotted');
        const t2 = t / 2;
        const [tx1, ty1, tx2, ty2] = [[t2, -t2, -t2, -t2], [-t2, t2, -t2, -t2], [t2, t2, -t2, t2], [t2, t2, t2, -t2]][i];
        const [A, B] = path;
        const x1 = A[0] + tx1 - this.dx, y1 = A[1] + ty1;
        const x2 = B[0] + tx2 - this.dx, y2 = B[1] + ty2;
        const W = Math.abs(i % 2 ? y2 - y1 : x2 - x1);
        const n = (dot ? Math.ceil(W / (2 * t)) : Math.ceil((W - t) / (4 * t)));
        const m = W / (4 * n + 1);
        const line = this.svg('line', {
            x1: this.fixed(x1), y1: this.fixed(y1),
            x2: this.fixed(x2), y2: this.fixed(y2),
            'stroke-width': this.fixed(t), stroke: color, 'stroke-linecap': dot ? 'round' : 'square',
            'stroke-dasharray': dot ? [1, this.fixed(W / n - .002)].join(' ') : [this.fixed(m), this.fixed(3 * m)].join(' ')
        });
        const adaptor = this.adaptor;
        const child = adaptor.firstChild(this.element);
        if (child) {
            adaptor.insert(line, child);
        }
        else {
            adaptor.append(this.element, line);
        }
    }
    handleAttributes() {
        const attributes = this.node.attributes;
        const defaults = attributes.getAllDefaults();
        const skip = SVGWrapper.skipAttributes;
        for (const name of attributes.getExplicitNames()) {
            if (skip[name] === false || (!(name in defaults) && !skip[name] &&
                !this.adaptor.hasAttribute(this.element, name))) {
                this.adaptor.setAttribute(this.element, name, attributes.getExplicit(name));
            }
        }
        if (attributes.get('class')) {
            const names = attributes.get('class').trim().split(/ +/);
            for (const name of names) {
                this.adaptor.addClass(this.element, name);
            }
        }
    }
    place(x, y, element = null) {
        x += this.dx;
        if (!(x || y))
            return;
        if (!element) {
            element = this.element;
            y = this.handleId(y);
        }
        const translate = `translate(${this.fixed(x)},${this.fixed(y)})`;
        const transform = this.adaptor.getAttribute(element, 'transform') || '';
        this.adaptor.setAttribute(element, 'transform', translate + (transform ? ' ' + transform : ''));
    }
    handleId(y) {
        if (!this.node.attributes || !this.node.attributes.get('id')) {
            return y;
        }
        const adaptor = this.adaptor;
        const h = this.getBBox().h;
        const children = adaptor.childNodes(this.element);
        children.forEach(child => adaptor.remove(child));
        const g = this.svg('g', { 'data-idbox': true, transform: `translate(0,${this.fixed(-h)})` }, children);
        adaptor.append(this.element, this.svg('text', { 'data-id-align': true }, [this.text('')]));
        adaptor.append(this.element, g);
        return y + h;
    }
    firstChild() {
        const adaptor = this.adaptor;
        let child = adaptor.firstChild(this.element);
        if (child && adaptor.kind(child) === 'text' && adaptor.getAttribute(child, 'data-id-align')) {
            child = adaptor.firstChild(adaptor.next(child));
        }
        if (child && adaptor.kind(child) === 'rect' && adaptor.getAttribute(child, 'data-hitbox')) {
            child = adaptor.next(child);
        }
        return child;
    }
    placeChar(n, x, y, parent, variant = null) {
        if (variant === null) {
            variant = this.variant;
        }
        const C = n.toString(16).toUpperCase();
        const [, , w, data] = this.getVariantChar(variant, n);
        if ('p' in data) {
            const path = (data.p ? 'M' + data.p + 'Z' : '');
            this.place(x, y, this.adaptor.append(parent, this.charNode(variant, C, path)));
        }
        else if ('c' in data) {
            const g = this.adaptor.append(parent, this.svg('g', { 'data-c': C }));
            this.place(x, y, g);
            x = 0;
            for (const n of this.unicodeChars(data.c, variant)) {
                x += this.placeChar(n, x, y, g, variant);
            }
        }
        else if (data.unknown) {
            const char = String.fromCodePoint(n);
            const text = this.adaptor.append(parent, this.jax.unknownText(char, variant));
            this.place(x, y, text);
            return this.jax.measureTextNodeWithCache(text, char, variant).w;
        }
        return w;
    }
    charNode(variant, C, path) {
        const cache = this.jax.options.fontCache;
        return (cache !== 'none' ? this.useNode(variant, C, path) : this.pathNode(C, path));
    }
    pathNode(C, path) {
        return this.svg('path', { 'data-c': C, d: path });
    }
    useNode(variant, C, path) {
        const use = this.svg('use', { 'data-c': C });
        const id = '#' + this.jax.fontCache.cachePath(variant, C, path);
        this.adaptor.setAttribute(use, 'href', id, svg_js_1.XLINKNS);
        return use;
    }
    drawBBox() {
        let { w, h, d } = this.getBBox();
        const box = this.svg('g', { style: {
                opacity: .25
            } }, [
            this.svg('rect', {
                fill: 'red',
                height: this.fixed(h),
                width: this.fixed(w)
            }),
            this.svg('rect', {
                fill: 'green',
                height: this.fixed(d),
                width: this.fixed(w),
                y: this.fixed(-d)
            })
        ]);
        const node = this.element || this.parent.element;
        this.adaptor.append(node, box);
    }
    html(type, def = {}, content = []) {
        return this.jax.html(type, def, content);
    }
    svg(type, def = {}, content = []) {
        return this.jax.svg(type, def, content);
    }
    text(text) {
        return this.jax.text(text);
    }
    fixed(x, n = 1) {
        return this.jax.fixed(x * 1000, n);
    }
}
exports.SVGWrapper = SVGWrapper;
SVGWrapper.kind = 'unknown';
SVGWrapper.borderFuzz = 0.005;
//# sourceMappingURL=Wrapper.js.map